from django.conf import settings
from django.contrib import admin

from payme.models import PaymeTransactions


class PaymeTransactionsUI(admin.ModelAdmin):
    """
    Custom admin interface for PaymeTransactions model.
    """
    list_display = ('pk', 'state', 'cancel_reason', 'created_at')
    list_filter = ('state', 'cancel_reason', 'created_at')
    search_fields = ('transaction_id', 'account_id')
    ordering = ('-created_at',)


if not getattr(settings, 'PAYME_DISABLE_ADMIN', False):
    admin.site.register(PaymeTransactions, PaymeTransactionsUI)



# Generated by Django 5.1.2 on 2024-12-22 05:15

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="PaymeTransactions",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("transaction_id", models.CharField(max_length=50)),
                ("account_id", models.BigIntegerField()),
                ("amount", models.DecimalField(decimal_places=2, max_digits=10)),
                (
                    "state",
                    models.IntegerField(
                        choices=[
                            (0, "Created"),
                            (1, "Initiating"),
                            (2, "Successfully"),
                            (-2, "Canceled after successful performed"),
                            (-1, "Canceled during initiation"),
                        ],
                        default=0,
                    ),
                ),
                ("cancel_reason", models.IntegerField(blank=True, null=True)),
                ("created_at", models.DateTimeField(auto_now_add=True, db_index=True)),
                ("updated_at", models.DateTimeField(auto_now=True, db_index=True)),
                (
                    "performed_at",
                    models.DateTimeField(blank=True, db_index=True, null=True),
                ),
                (
                    "cancelled_at",
                    models.DateTimeField(blank=True, db_index=True, null=True),
                ),
            ],
            options={
                "verbose_name": "Payme Transaction",
                "verbose_name_plural": "Payme Transactions",
                "db_table": "payme_transactions",
                "ordering": ["-created_at"],
            },
        ),
    ]



import typing as t
from dataclasses import dataclass


@dataclass
class Common:
    """
    The common response structure.
    """

    @classmethod
    def from_dict(cls, data: t.Dict):
        """
        Prepare fields for nested dataclasses
        """
        field_values = {}
        for field in cls.__dataclass_fields__:
            field_type = cls.__dataclass_fields__[field].type
            field_data = data.get(field)

            if isinstance(field_data, dict) and issubclass(field_type, Common):
                field_values[field] = field_type.from_dict(field_data)
            else:
                field_values[field] = field_data

        return cls(**field_values)


@dataclass
class Card(Common):
    """
    The card object represents a credit card.
    """

    number: str
    expire: str
    token: str
    recurrent: bool
    verify: bool
    type: str
    number_hash: t.Optional[str] = None


@dataclass
class Result(Common):
    """
    The result object contains the created card.
    """

    card: Card


@dataclass
class CardsCreateResponse(Common):
    """
    The cards.create response.
    """

    jsonrpc: str
    result: Result


@dataclass
class VerifyResult(Common):
    """
    The result object for the verification response.
    """

    sent: bool
    phone: str
    wait: int


@dataclass
class GetVerifyResponse(Common):
    """
    The verification response structure.
    """

    jsonrpc: str
    result: VerifyResult


@dataclass
class VerifyResponse(Common):
    """
    The verification response structure.
    """

    jsonrpc: str
    result: Result


@dataclass
class RemoveCardResult(Common):
    """
    The result object for the removal response.
    """

    success: bool


@dataclass
class RemoveResponse(Common):
    """
    The remove response structure.
    """

    jsonrpc: str
    result: RemoveCardResult


@dataclass
class CheckResponse(Common):
    """
    The check response structure.
    """

    jsonrpc: str
    result: Result



import base64
import binascii
import logging
from decimal import Decimal

from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.module_loading import import_string
from rest_framework import views
from rest_framework.response import Response

from payme import exceptions
from payme.models import PaymeTransactions
from payme.types import response
from payme.util import time_to_payme, time_to_service

logger = logging.getLogger(__name__)
AccountModel = import_string(settings.PAYME_ACCOUNT_MODEL)


def handle_exceptions(func):
    """
    Decorator to handle exceptions and raise appropriate Payme exceptions.
    """
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except KeyError as exc:
            message = "Invalid parameters received."
            logger.error(f"{message}: {exc}s {exc} {args} {kwargs}")
            raise exceptions.InternalServiceError(message) from exc

        except AccountModel.DoesNotExist as exc:
            logger.error(f"Account does not exist: {exc} {args} {kwargs}")
            raise exceptions.AccountDoesNotExist(str(exc)) from exc

        except ValidationError as exc:
            logger.error(f"Invalid account identifier {exc}")
            raise exceptions.AccountDoesNotExist("Invalid account identifier.")

        except PaymeTransactions.DoesNotExist as exc:
            logger.error(f"Transaction does not exist: {exc} {args} {kwargs}")
            raise exceptions.AccountDoesNotExist(str(exc)) from exc

        except exceptions.exception_whitelist as exc:
            # No need to raise exception for exception whitelist
            raise exc
        except Exception as exc:
            logger.error(f"Unexpected error: {exc} {args} {kwargs}")
            raise exceptions.InternalServiceError(str(exc)) from exc

    return wrapper


class PaymeWebHookAPIView(views.APIView):
    """
    A webhook view for Payme.
    """
    authentication_classes = ()

    def post(self, request, *args, **kwargs):
        """
        Handle the incoming webhook request.
        """
        self.check_authorize(request)

        payme_methods = {
            "GetStatement": self.get_statement,
            "CancelTransaction": self.cancel_transaction,
            "PerformTransaction": self.perform_transaction,
            "CreateTransaction": self.create_transaction,
            "CheckTransaction": self.check_transaction,
            "CheckPerformTransaction": self.check_perform_transaction,
            "SetFiscalData": self.set_fiscal_data,
        }

        try:
            method = request.data["method"]
            params = request.data["params"]
        except KeyError as exc:
            message = f"Error processing webhook: {exc}"
            raise exceptions.InternalServiceError(message) from exc

        if method in payme_methods:
            result = payme_methods[method](params)
            return Response(result)

        raise exceptions.MethodNotFound("Method not supported yet!")

    @staticmethod
    def check_authorize(request):
        """
        Verify the integrity of the request using the merchant key.
        """
        password = request.META.get('HTTP_AUTHORIZATION')
        if not password:
            raise exceptions.PermissionDenied("Missing authentication credentials")

        password = password.split()[-1]

        try:
            password = base64.b64decode(password).decode('utf-8')
        except (binascii.Error, UnicodeDecodeError) as exc:
            raise exceptions.PermissionDenied("Decoding error in authentication credentials") from exc

        try:
            payme_key = password.split(':')[-1]
        except IndexError as exc:
            message = "Invalid merchant key format in authentication credentials"
            raise exceptions.PermissionDenied(message) from exc

        if payme_key != settings.PAYME_KEY:
            raise exceptions.PermissionDenied("Invalid merchant key specified")

    @handle_exceptions
    def fetch_account(self, params: dict):
        """
        Fetch account based on settings and params.
        """
        account_value = params["account"].get(settings.PAYME_ACCOUNT_FIELD)
        if not account_value:
            raise exceptions.InvalidAccount("Missing account field in parameters.")

        account = AccountModel.objects.get(pk=account_value)

        return account

    @handle_exceptions
    def validate_amount(self, account, amount):
        """
        Validates if the amount matches for one-time payment accounts.
        """
        if not settings.PAYME_ONE_TIME_PAYMENT:
            return True

        expected_amount = Decimal(getattr(account, settings.PAYME_AMOUNT_FIELD)) * 100
        received_amount = Decimal(amount)

        if expected_amount != received_amount:
            raise exceptions.IncorrectAmount(
                f"Invalid amount. Expected: {expected_amount}, received: {received_amount}"
            )

        return True

    @handle_exceptions
    def check_perform_transaction(self, params) -> response.CheckPerformTransaction:
        """
        Handle the pre_create_transaction action.
        """
        account = self.fetch_account(params)
        self.validate_amount(account, params.get('amount'))

        result = response.CheckPerformTransaction(allow=True)
        return result.as_resp()

    @handle_exceptions
    def create_transaction(self, params) -> response.CreateTransaction:
        """
        Handle the create_transaction action.
        """
        transaction_id = params["id"]
        amount = Decimal(params.get('amount', 0))
        account = self.fetch_account(params)

        self.validate_amount(account, amount)

        defaults = {
            "amount": amount,
            "state": PaymeTransactions.INITIATING,
            "account_id": account.pk,
        }

        # Handle already existing transaction with the same ID for one-time payments
        if settings.PAYME_ONE_TIME_PAYMENT:
            # Check for an existing transaction with a different transaction_id for the given account
            if (
                PaymeTransactions.objects.filter(account_id=account.pk)
                .exclude(transaction_id=transaction_id)
                .exists()
            ):
                message = f"Transaction {transaction_id} already exists (Payme)."
                logger.warning(message)
                raise exceptions.TransactionAlreadyExists(message)

        transaction, _ = PaymeTransactions.objects.get_or_create(
            transaction_id=transaction_id,
            defaults=defaults
        )

        result = response.CreateTransaction(
            transaction=transaction.transaction_id,
            state=transaction.state,
            create_time=time_to_payme(transaction.created_at),
        )
        result = result.as_resp()

        # callback event
        self.handle_created_payment(params, result)

        return result

    @handle_exceptions
    def perform_transaction(self, params) -> response.PerformTransaction:
        """
        Handle the successful payment.
        """
        transaction = PaymeTransactions.get_by_transaction_id(transaction_id=params["id"])

        if transaction.is_performed():
            result = response.PerformTransaction(
                transaction=transaction.transaction_id,
                state=transaction.state,
                perform_time=time_to_payme(transaction.performed_at),
            )
            return result.as_resp()

        transaction.mark_as_performed()

        result = response.PerformTransaction(
            transaction=transaction.transaction_id,
            state=transaction.state,
            perform_time=time_to_payme(transaction.performed_at),
        )
        result = result.as_resp()

        # callback successfully event
        self.handle_successfully_payment(params, result)

        return result

    @handle_exceptions
    def check_transaction(self, params: dict) -> dict | str | response.CheckPerformTransaction:
        """
        Handle check transaction request.
        """
        transaction = PaymeTransactions.get_by_transaction_id(transaction_id=params["id"])

        result = response.CheckTransaction(
            transaction=transaction.transaction_id,
            state=transaction.state,
            reason=transaction.cancel_reason,
            create_time=time_to_payme(transaction.created_at),
            perform_time=time_to_payme(transaction.performed_at),
            cancel_time=time_to_payme(transaction.cancelled_at),
        )

        return result.as_resp()

    @handle_exceptions
    def cancel_transaction(self, params) -> response.CancelTransaction:
        """
        Handle the cancelled payment.
        """
        transaction = PaymeTransactions.get_by_transaction_id(transaction_id=params["id"])

        if transaction.is_cancelled():
            return self._cancel_response(transaction)

        if transaction.is_performed():
            transaction.mark_as_cancelled(
                cancel_reason=params["reason"],
                state=PaymeTransactions.CANCELED
            )
        elif transaction.is_created_in_payme():
            transaction.mark_as_cancelled(
                cancel_reason=params["reason"],
                state=PaymeTransactions.CANCELED_DURING_INIT
            )

        result = self._cancel_response(transaction)

        # callback cancelled transaction event
        self.handle_cancelled_payment(params, result)

        return result

    @handle_exceptions
    def get_statement(self, params) -> response.GetStatement:
        """
        Retrieves a statement of transactions.
        """
        date_range = [time_to_service(params['from']), time_to_service(params['to'])]

        transactions = PaymeTransactions.objects.filter(
            created_at__range=date_range
        ).order_by('-created_at')

        result = response.GetStatement(transactions=[])

        for transaction in transactions:
            result.transactions.append({
                "transaction": transaction.transaction_id,
                "amount": transaction.amount,
                "account": {
                    settings.PAYME_ACCOUNT_FIELD: transaction.account_id
                },
                "reason": transaction.cancel_reason,
                "state": transaction.state,
                "create_time": time_to_payme(transaction.created_at),
                "perform_time": time_to_payme(transaction.performed_at),
                "cancel_time": time_to_payme(transaction.cancelled_at),
                "time": time_to_payme(transaction.created_at),
            })

        return result.as_resp()

    @handle_exceptions
    def set_fiscal_data(self, params):
        """
        Set fiscal data for the given transaction.
        """
        transaction = PaymeTransactions.get_by_transaction_id(transaction_id=params["id"])

        fiscal_data = params.get("fiscal_data")
        if not fiscal_data:
            raise exceptions.InvalidFiscalParams(
                "Missing fiscal_data field in parameters."
            )

        fiscal_type = params.get("type")

        if fiscal_type not in ("PERFORM", "CANCEL"):
            raise exceptions.InvalidFiscalParams(
                f"Invalid fiscal type. Expected 'PERFORM' or 'CANCEL', got: {fiscal_type}"
            )

        fiscal_data["type"] = fiscal_type
        transaction.fiscal_data = fiscal_data
        transaction.save()

        result = response.SetFiscalData(success=True)
        return result.as_resp()

    def _cancel_response(self, transaction):
        """
        Helper method to generate cancel transaction response.
        """
        result = response.CancelTransaction(
            transaction=transaction.transaction_id,
            state=transaction.state,
            cancel_time=time_to_payme(transaction.cancelled_at),
        )
        return result.as_resp()

    def handle_pre_payment(self, params, result, *args, **kwargs):
        """
        Handle the pre_create_transaction action. You can override this method
        """
        print(f"Transaction pre_created for this params: {params} and pre_created_result: {result}")

    def handle_created_payment(self, params, result, *args, **kwargs):
        """
        Handle the successful payment. You can override this method
        """
        print(f"Transaction created for this params: {params} and cr_result: {result}")

    def handle_successfully_payment(self, params, result, *args, **kwargs):
        """
        Handle the successful payment. You can override this method
        """
        print(f"Transaction successfully performed for this params: {params} and performed_result: {result}")

    def handle_cancelled_payment(self, params, result, *args, **kwargs):
        """
        Handle the cancelled payment. You can override this method
        """
        print(f"Transaction cancelled for this params: {params} and cancelled_result: {result}")



import typing as t
from dataclasses import dataclass, field


class CommonResponse:
    """
    The common response structure
    """

    def as_resp(self):
        response = {"result": {}}
        for key, value in self.__dict__.items():
            response["result"][key] = value
        return response


@dataclass
class Shipping(CommonResponse):
    """
    Shipping information response structure
    """

    title: str
    price: int


@dataclass
class Item(CommonResponse):
    """
    Item information response structure
    """

    discount: int
    title: str
    price: int
    count: int
    code: str
    units: int
    vat_percent: int
    package_code: str

    def as_resp(self):
        return {
            "discount": self.discount,
            "title": self.title,
            "price": self.price,
            "count": self.count,
            "code": self.code,
            "units": self.units,
            "vat_percent": self.vat_percent,
            "package_code": self.package_code,
        }


@dataclass
class CheckPerformTransaction(CommonResponse):
    """
    Receipt information response structure for transaction checks.
    """

    allow: bool
    additional: t.Optional[t.Dict[str, str]] = None
    receipt_type: t.Optional[int] = None
    shipping: t.Optional[Shipping] = None
    items: t.List[Item] = field(default_factory=list)

    def add_item(self, item: Item):
        self.items.append(item)

    def as_resp(self):
        detail_dict = {}
        receipt_dict = {"allow": self.allow}

        if self.additional:
            receipt_dict["additional"] = self.additional

        if isinstance(self.receipt_type, int):
            detail_dict["receipt_type"] = self.receipt_type

        if self.shipping:
            detail_dict["shipping"] = self.shipping.as_resp()

        if self.items:
            detail_dict["items"] = [item.as_resp() for item in self.items]

        if detail_dict:
            receipt_dict["detail"] = detail_dict

        return {"result": receipt_dict}


@dataclass
class CreateTransaction(CommonResponse):
    """
    The create transaction request
    """

    transaction: str
    state: str
    create_time: int


@dataclass
class PerformTransaction(CommonResponse):
    """
    The perform transaction response
    """

    transaction: str
    state: str
    perform_time: int


@dataclass
class CancelTransaction(CommonResponse):
    """
    The cancel transaction request
    """

    transaction: str
    state: str
    cancel_time: str


@dataclass
class CheckTransaction(CommonResponse):
    """
    The check transaction request
    """

    transaction: str
    state: str
    reason: str
    create_time: int
    perform_time: t.Optional[int] = None
    cancel_time: t.Optional[int] = None


@dataclass
class GetStatement(CommonResponse):
    """
    The check perform transactions response
    """

    transactions: t.List[t.Dict[str, str | int | t.Dict[str, str | int]]]


@dataclass
class SetFiscalData(CommonResponse):
    """
    The set fiscal data request
    """

    success: bool



# Generated by Django 5.1.2 on 2025-03-14 08:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("payme", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="paymetransactions",
            name="fiscal_data",
            field=models.JSONField(blank=True, null=True),
        ),
    ]



import typing as t

from payme.const import Networks
from payme.classes.cards import Cards
from payme.classes.receipts import Receipts
from payme.classes.initializer import Initializer


class Payme:
    """
    The payme class provides a simple interface
    """

    def __init__(
        self,
        payme_id: str,
        fallback_id: t.Optional[str] = None,
        payme_key: t.Optional[str] = None,
        is_test_mode: bool = False,
    ) -> None:
        # initialize payme network
        url = Networks.PROD_NET.value

        if is_test_mode is True:
            url = Networks.TEST_NET.value

        self.cards = Cards(url=url, payme_id=payme_id)
        self.initializer = Initializer(
            payme_id=payme_id, fallback_id=fallback_id, is_test_mode=is_test_mode
        )
        if payme_key:
            self.receipts = Receipts(url=url, payme_id=payme_id, payme_key=payme_key)



# Generated migration to increase amount field precision

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("payme", "0004_alter_paymetransactions_account_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymetransactions",
            name="amount",
            field=models.DecimalField(decimal_places=2, max_digits=15),
        ),
    ]




"""
This module contains models and functionality for tracking changes in payment transactions.
It logs any significant modifications to payment transactions such as amount, state, or payment method,
allowing for a detailed historical record of each transaction's state over time.
"""
from django.db import models
from django.utils import timezone


class PaymeTransactions(models.Model):
    """
    Model to store payment transactions.
    """
    CREATED = 0
    INITIATING = 1
    SUCCESSFULLY = 2
    CANCELED = -2
    CANCELED_DURING_INIT = -1

    STATE = [
        (CREATED, "Created"),
        (INITIATING, "Initiating"),
        (SUCCESSFULLY, "Successfully"),
        (CANCELED, "Canceled after successful performed"),
        (CANCELED_DURING_INIT, "Canceled during initiation"),
    ]

    transaction_id = models.CharField(max_length=50)
    account_id = models.CharField(max_length=256, null=False)
    amount = models.DecimalField(max_digits=15, decimal_places=2)
    state = models.IntegerField(choices=STATE, default=CREATED)
    fiscal_data = models.JSONField(default=dict)
    cancel_reason = models.IntegerField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True, db_index=True)
    updated_at = models.DateTimeField(auto_now=True, db_index=True)
    performed_at = models.DateTimeField(null=True, blank=True, db_index=True)
    cancelled_at = models.DateTimeField(null=True, blank=True, db_index=True)

    class Meta:
        """
        Model Meta options.
        """
        verbose_name = "Payme Transaction"
        verbose_name_plural = "Payme Transactions"
        ordering = ["-created_at"]
        db_table = "payme_transactions"

    def __str__(self):
        """
        String representation of the PaymentTransaction model.
        """
        return f"Payme Transaction #{self.transaction_id} Account: {self.account_id} - {self.state}"

    @classmethod
    def get_by_transaction_id(cls, transaction_id):
        """
        Class method to get a PaymentTransaction instance by its transaction ID.

        :param transaction_id: The unique ID of the transaction.
        :return: The PaymentTransaction instance or None if not found.
        """
        return cls.objects.get(transaction_id=transaction_id)

    def is_performed(self) -> bool:
        """
        Check if the transaction is completed.

        :return: True if the transaction is completed, False otherwise.
        """
        return self.state == self.SUCCESSFULLY

    def is_cancelled(self) -> bool:
        """
        Check if the transaction is cancelled.

        :return: True if the transaction is cancelled, False otherwise.
        """
        return self.state in [
            self.CANCELED,
            self.CANCELED_DURING_INIT
        ]

    def is_created(self) -> bool:
        """
        Check if the transaction is created.

        :return: True if the transaction is created, False otherwise.
        """
        return self.state == self.CREATED

    def is_created_in_payme(self) -> bool:
        """
        Check if the transaction was created in Payme.

        :return: True if the transaction was created in Payme, False otherwise.
        """
        return self.state == self.INITIATING

    def mark_as_cancelled(self, cancel_reason: int, state: int) -> "PaymeTransactions":
        """
        Mark the transaction as cancelled.

        :param cancel_reason: The reason for cancelling the transaction.
        :return: True if the transaction was successfully marked as cancelled, False otherwise.
        """
        if self.state == state:
            return self

        self.state = state
        self.cancel_reason = cancel_reason
        self.cancelled_at = timezone.now()
        self.save()
        return self

    def mark_as_performed(self) -> bool:
        """
        Mark the transaction as performed.

        :return: True if the transaction was successfully marked as performed, False otherwise.
        """
        if self.state != self.INITIATING:
            return False

        self.state = self.SUCCESSFULLY
        self.performed_at = timezone.now()
        self.save()
        return True



# Generated by Django 5.2.5 on 2025-08-25 19:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("payme", "0003_alter_paymetransactions_fiscal_data"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymetransactions",
            name="account_id",
            field=models.CharField(max_length=256),
        ),
    ]



import base64

from django.conf import settings


class Initializer:
    """
    Initialize the Payme class with necessary details.

    Attributes
    ----------
    payme_id: str
        The Payme ID associated with your account
    """

    def __init__(
        self, payme_id: str = None, fallback_id: str = None, is_test_mode: bool = False
    ) -> None:
        self.payme_id = payme_id
        self.fallback_id = fallback_id
        self.is_test_mode = is_test_mode

    def generate_pay_link(self, id: int, amount: int, return_url: str) -> str:
        """
        Generate a payment link for a specific order.

        This method encodes the payment parameters into a base64 string and
        constructs a URL for the Payme checkout.

        Parameters
        ----------
        id : int
            Unique identifier for the account.
        amount : int
            The amount associated with the order in currency units.
        return_url : str
            The URL to which the user will be redirected after the payment is
            processed.

        Returns
        -------
        str
            A payment link formatted as a URL, ready to be used in the payment
            process.

        References
        ----------
        For full method documentation, visit:
        https://developer.help.paycom.uz/initsializatsiya-platezhey/
        """
        amount = amount * 100  # Convert amount to the smallest currency unit
        params = f"m={self.payme_id};ac.{settings.PAYME_ACCOUNT_FIELD}={id};a={amount};c={return_url}"
        params = base64.b64encode(params.encode("utf-8")).decode("utf-8")

        if self.is_test_mode is True:
            return f"https://test.paycom.uz/{params}"

        return f"https://checkout.paycom.uz/{params}"

    def generate_fallback_link(self, form_fields: dict = None):
        """
        Generate a fallback URL for the Payme checkout.

        Parameters
        ----------
        fields : dict, optional
            Additional query parameters to be appended to the fallback URL.

        Returns
        -------
        str
            A fallback URL formatted as a URL, ready to be used in the payment
            process.
        """
        result = f"https://payme.uz/fallback/merchant/?id={self.fallback_id}"

        if form_fields is not None:
            for key, value in form_fields.items():
                result += f"&{key}={value}"

        return result



import typing as t
from dataclasses import dataclass


@dataclass
class Common:
    """
    The common response structure.
    """

    jsonrpc: str
    id: int

    @classmethod
    def from_dict(cls, data: t.Dict):
        """
        Prepare fields for nested dataclasses
        """
        field_values = {}
        for field in cls.__dataclass_fields__:
            field_type = cls.__dataclass_fields__[field].type
            field_data = data.get(field)

            if isinstance(field_data, dict) and issubclass(field_type, Common):
                field_values[field] = field_type.from_dict(field_data)
            else:
                field_values[field] = field_data

        return cls(**field_values)


@dataclass
class Account(Common):
    """
    The account object represents a user's banking account.
    """

    _id: str
    account_number: str
    account_name: str
    account_type: str
    bank_name: str
    currency: str
    status: str


@dataclass
class PaymentMethod(Common):
    """
    The payment method object represents a user's payment method.
    """

    name: str
    title: str
    value: str
    main: t.Optional[bool] = None


@dataclass
class Detail(Common):
    """
    The detail object represents additional details for a receipt.
    """

    discount: t.Optional[str] = None
    shipping: t.Optional[str] = None
    items: t.Optional[str] = None


# pylint: disable=C0103
@dataclass
class MerchantEpos(Common):
    """
    The merchantEpos object represents a user's ePOS.
    """

    eposId: str
    eposName: str
    eposType: str
    eposTerminalId: str


@dataclass
class Meta(Common):
    """
    The meta object represents additional metadata for a receipt.
    """

    source: t.Any = None
    owner: t.Any = None
    host: t.Any = None


@dataclass
class Merchant:
    """
    The merchant object represents a user's merchant.
    """

    _id: str
    name: str
    organization: str
    address: t.Optional[str] = None
    business_id: t.Optional[str] = None
    epos: t.Optional[MerchantEpos] = None
    restrictions: t.Optional[str] = None
    date: t.Optional[int] = None
    logo: t.Optional[str] = None
    type: t.Optional[str] = None
    terms: t.Optional[str] = None


@dataclass
class Payer(Common):
    """
    The payer object represents a user's payer.
    """

    phone: str


@dataclass
class Receipt(Common):
    """
    The receipt object represents a payment receipt.
    """

    _id: str
    create_time: int
    pay_time: int
    cancel_time: int
    state: int
    type: int
    external: bool
    operation: int
    error: t.Any = None
    description: t.Optional[str] = None
    detail: t.Optional[Detail] = None
    currency: t.Optional[int] = None
    commission: t.Optional[int] = None
    card: t.Optional[str] = None
    creator: t.Optional[str] = None
    payer: t.Optional[Payer] = None
    amount: t.Optional[t.Union[float, int]] = None
    account: t.Optional[t.List[Account]] = None
    merchant: t.Optional[Merchant] = None
    processing_id: t.Optional[str] = None
    meta: t.Optional[Meta] = None


@dataclass
class CreateResult(Common):
    """
    The result object for the create response.
    """

    receipt: Receipt


@dataclass
class CreateResponse(Common):
    """
    The create response structure.
    """

    result: CreateResult


@dataclass
class PayResponse(CreateResponse):
    """
    The pay response structure.
    """


@dataclass
class SendResult(Common):
    """
    The result object for the send response.
    """

    success: bool


@dataclass
class SendResponse(Common):
    """
    The send response structure.
    """

    result: SendResult


@dataclass
class CancelResponse(CreateResponse):
    """
    The cancel response structure.
    """


@dataclass
class CheckResult(Common):
    """
    The result object for the check response.
    """

    state: int


@dataclass
class CheckResponse(Common):
    """
    The check response structure.
    """

    result: CheckResult


@dataclass
class GetResponse(CreateResponse):
    """
    The result object for the get response.
    """


@dataclass
class GetAllResponse(Common):
    """
    The result object for the get all response.
    """
    result: list[Receipt] = None


@dataclass
class SetFiscalDataResponse(Common):
    """
    The result object for the set_fiscal_data response.
    """
    result: SendResult



"""
Payme enumerations
"""
from enum import Enum


class Networks(str, Enum):
    """
    Payme networks
    """
    PROD_NET = "https://checkout.paycom.uz/api"
    TEST_NET = "https://checkout.test.paycom.uz/api"



import pathlib

from setuptools import find_packages, setup

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name='payme-sdk',
    version='0.0.1',
    license='MIT',
    author="Jahongir Hakimjonov",
    author_email='jahongirhakimjonov@gmail.com',
    packages=find_packages(),
    url='https://github.com/JahongirHakimjonov/payme-sdk',
    keywords=['payme-sdk'],
    install_requires=[
        'requests==2.*',
        "dataclasses==0.*;python_version<'3.7'",  # will only install on py3.6
        'djangorestframework==3.*'
      ],
    long_description=long_description,
    long_description_content_type="text/markdown",
)



from datetime import datetime


def time_to_payme(datatime) -> int:
    """
    Convert datetime object to Payme's datetime format.

    Payme's datetime format is in the format: YYYY-MM-DD HH:MM:SS.ssssss

    Args:
    datatime (datetime): The datetime object to convert.

    Returns:
    str: The datetime object in Payme's datetime format.
    """
    if not datatime:
        return 0

    return int(datatime.timestamp() * 1000)


def time_to_service(milliseconds: int) -> datetime:
    """
    Converts milliseconds since the epoch to a datetime object.
    """
    return datetime.fromtimestamp(milliseconds / 1000)
